@@ -514,7 +514,7 @@ void file_open_cb(GtkWidget *widget, gpointer cbdata)
 	g_assert(w != NULL);
 
 	if (w->open_fileselector == NULL) {
-		w->open_fileselector = gtk_file_selection_new("Open File...");
+		w->open_fileselector = gtk_file_selection_new(_("Open File..."));
 		gtk_file_selection_hide_fileop_buttons(
 			GTK_FILE_SELECTION(w->open_fileselector));
 		gtk_signal_connect(GTK_OBJECT(w->open_fileselector), "destroy",
@@ -585,7 +585,7 @@ file_save_cb(GtkWidget *widget, gpointer cbdata)
 	 if ((gE_file_save(data->window, gE_document_current(data->window),
 	               gE_document_current(data->window)->filename)) != 0)
 	   {
-	gE_msgbar_set(data->window, "Read only file!");
+	gE_msgbar_set(data->window, _("Read only file!"));
 	file_save_as_cb(NULL, data);
         }
 
@@ -599,7 +599,7 @@ file_save_as_cb(GtkWidget *widget, gpointer cbdata)
 
 	g_assert(data != NULL);
 
-	safs = gtk_file_selection_new("Save As...");
+	safs = gtk_file_selection_new(_("Save As..."));
 
 	data->temp1 = safs;
 	gtk_signal_connect(GTK_OBJECT(safs), "destroy",
@@ -852,7 +852,9 @@ file_quit_cb(GtkWidget *widget, gpointer cbdata)
 	gE_data *data = (gE_data *)cbdata;
 
 	g_assert(data != NULL);
-
+	
+	plugin_save_list();
+	
 	while (window_list) {
 		data->window = g_list_nth_data(window_list, 0);
 		window_close_cb(widget, data);
@@ -875,7 +877,7 @@ edit_cut_cb(GtkWidget *widget, gpointer cbdata)
 		GTK_EDITABLE(gE_document_current(data->window)->text));
 #else
 	gtk_editable_cut_clipboard(GTK_EDITABLE(
-		gE_document_current(data->window)->text), GDK_CURRENT_TIME);
+		gE_document_current(data->window)->text));
 #endif
 	gE_msgbar_set(data->window, MSGBAR_CUT);
 }
