@@ -20,12 +20,18 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <gtk/gtk.h>
+#include <gnome.h>
 #include "plugin.h"
 
+GList *plugin_list = NULL;
+plugin_callback_struct pl_callbacks;
+
+
 /* static void process_command( plugin *plug, gchar *buffer, int length, gpointer data ); */
 /* static void *plugin_parse(plugin *plug); */
 static void
 process_command( plugin *plug, gchar *buffer, int length, gpointer data );
+plugin *custom_plugin_new_with_query( gchar *path, gchar *plugin_name, gboolean query );
 
 typedef struct
 {
@@ -39,21 +45,48 @@ typedef struct
 } partly_read;
 
 plugin *plugin_new_with_query( gchar *plugin_name, gboolean query )
+{
+	   
+	return custom_plugin_new_with_query ( "", plugin_name, query );
+	
+}
+
+plugin *plugin_new( gchar *plugin_name )
+{
+  return plugin_new_with_query( plugin_name, FALSE );
+}
+
+plugin *plugin_query( gchar *plugin_name )
+{
+  return plugin_new_with_query( plugin_name, TRUE );
+}
+
+plugin *custom_plugin_new_with_query( gchar *path, gchar *plugin_name, gboolean query )
 {
   int toline[2]; /* Commands to the plugin. */
   int fromline[2]; /* Commands from the plugin. */
   int dataline[2]; /* Data to the plugin. */
   plugin *new_plugin = g_new( plugin, 1 ); /* The plugin. */
+  DIR *dir;
+  struct dirent *direntry;
+  gchar *temp, *long_name;
   
+ 
   if ( pipe( toline ) == -1 || pipe( fromline ) == -1 || pipe( dataline ) == -1 )
     {
       g_free( new_plugin );
       return 0;
     }
+    
+  
+  	  temp = plugin_name;
+	  plugin_name = g_malloc0( strlen( temp ) + strlen( path ) + 2 );
+	  sprintf( plugin_name, "%s/%s", path, temp );
+      
   new_plugin->pipe_to = toline[1];
   new_plugin->pipe_from = fromline[0];
   new_plugin->pipe_data = dataline[1];
-  new_plugin->name = g_strdup( strrchr( plugin_name, '/' ) ? strrchr( plugin_name, '/' ) + 1 : plugin_name );
+ new_plugin->name = g_strdup( plugin_name );
   new_plugin->pid = fork();
   if ( new_plugin->pid == 0 )
     {
@@ -79,12 +112,13 @@ plugin *plugin_new_with_query( gchar *plugin_name, gboolean query )
 	}
       else
 	argv[5] = NULL;
-      if ( *plugin_name != '/' )
-	{
-	  gchar *temp = plugin_name;
-	  plugin_name = g_malloc0( strlen( temp ) + strlen( PLUGINDIR ) + 2 );
-	  sprintf( plugin_name, "%s/%s", PLUGINDIR, temp );
-	}
+     /* if ( plugin_name != '/' )
+	{ 
+	  temp = plugin_name;
+	  plugin_name = g_malloc0( strlen( temp ) + strlen( path ) + 2 );
+	  sprintf( plugin_name, "%s/%s", path, temp );
+	  new_plugin->name = g_strdup( plugin_name ); 
+	}*/
       execv(plugin_name, argv);
       /* This is only reached if something goes wrong. */
       _exit( 1 );
@@ -103,32 +137,62 @@ plugin *plugin_new_with_query( gchar *plugin_name, gboolean query )
   return new_plugin;
 }
 
-plugin *plugin_new( gchar *plugin_name )
+plugin *custom_plugin_new( gchar *path, gchar *plugin_name )
 {
-  return plugin_new_with_query( plugin_name, FALSE );
+  return custom_plugin_new_with_query( path, plugin_name, FALSE );
 }
 
-plugin *plugin_query( gchar *plugin_name )
+void custom_plugin_query( gchar *path, gchar *plugin_name, plugin_callback_struct *callbacks )
 {
-  return plugin_new_with_query( plugin_name, TRUE );
+plugin *plug;
+plugin_list_data *pl_list;
+int i;
+
+  plug = custom_plugin_new_with_query( path, plugin_name, TRUE );
+  plug->callbacks = *callbacks;
+  
+#if 0 		
+	      plug->context = 0;
+	      pthread_create( &plug->thread, NULL, (void *(*)(void *)) plugin_parse, plug );
+#else
+	      plugin_get_all( plug, 1, process_command, NULL ); 
+	      plugin_send_int( plug, 0 );
+#endif
+
+	if ((pl_list = g_malloc0 (sizeof (plugin_list_data))) == NULL)
+	  {
+	    g_print ("custom_plugin_query: Memory Allocation Error.\n");
+	    return;
+	  }
+	  
+	pl_list->name = g_strdup (plugin_name);
+	pl_list->location = g_strdup (path);
+	plugin_list = g_list_append (plugin_list, pl_list);
 }
 
-plugin *plugin_new_with_param( gchar *plugin_name, int argc, gchar *arg[] )
+plugin *custom_plugin_new_with_param( gchar *path, gchar *plugin_name, int argc, gchar *arg[] )
 {
   int toline[2]; /* Commands to the plugin. */
   int fromline[2]; /* Commands from the plugin. */
   int dataline[2]; /* Data to the plugin. */
   plugin *new_plugin = g_new( plugin, 1 ); /* The plugin. */
+  gchar *temp;
   
   if ( pipe( toline ) == -1 || pipe( fromline ) == -1 || pipe( dataline ) == -1 )
     {
       g_free( new_plugin );
       return 0;
     }
+    
+    
+      	  temp = plugin_name;
+	  plugin_name = g_malloc0( strlen( temp ) + strlen( path ) + 2 );
+	  sprintf( plugin_name, "%s/%s", path, temp );
+	  
   new_plugin->pipe_to = toline[1];
   new_plugin->pipe_from = fromline[0];
   new_plugin->pipe_data = dataline[1];
-  new_plugin->name = g_strdup( strrchr( plugin_name, '/' ) ? strrchr( plugin_name, '/' ) + 1 : plugin_name );
+  new_plugin->name = g_strdup( plugin_name );
   new_plugin->pid = fork();
   if ( new_plugin->pid == 0 )
     {
@@ -153,12 +217,13 @@ plugin *plugin_new_with_param( gchar *plugin_name, int argc, gchar *arg[] )
 	  argv[i + 5] = arg[i];
 	}
       argv[5+argc] = NULL;
-      if ( *plugin_name != '/' )
+    /*  if ( *plugin_name != '/' )
 	{
 	  gchar *temp = plugin_name;
-	  plugin_name = g_malloc0( strlen( temp ) + strlen( PLUGINDIR ) + 2 );
-	  sprintf( plugin_name, "%s/%s", PLUGINDIR, temp );
-	}
+	  plugin_name = g_malloc0( strlen( temp ) + strlen( path ) + 2 );
+	  sprintf( plugin_name, "%s/%s", path, temp );
+	}*/
+	
       execv(plugin_name, argv);
       /* This is only reached if something goes wrong. */
       _exit( 1 );
@@ -177,12 +242,28 @@ plugin *plugin_new_with_param( gchar *plugin_name, int argc, gchar *arg[] )
   return new_plugin;
 }
 
+plugin *plugin_new_with_param( gchar *plugin_name, int argc, gchar *arg[] )
+{
+	   
+	return custom_plugin_new_with_param ( PLUGINDIR, plugin_name, argc, arg );
+	
+}
+
 void plugin_query_all( plugin_callback_struct *callbacks )
 {
-  DIR *dir = opendir( PLUGINDIR );
-  struct dirent *direntry;
-  gchar *shortname;
+	   
+	  custom_plugin_query_all ( PLUGINDIR, callbacks );
+	
+}
 
+void custom_plugin_query_all( gchar *path, plugin_callback_struct *callbacks )
+{
+  DIR *dir = opendir( path );
+  struct dirent *direntry;
+  gchar *shortname, *long_name;
+  plugin_list_data *pl_list;
+
+	  
   if ( dir )
     {
       while ( ( direntry = readdir( dir ) ) )
@@ -193,8 +274,12 @@ void plugin_query_all( plugin_callback_struct *callbacks )
 	  else
 	    shortname = direntry->d_name;     
 	  if ( strcmp( shortname, "." ) && strcmp( shortname, ".." ) )
-	    {
-	      plug = plugin_query( direntry->d_name );
+   	    {
+   	      if ( *direntry->d_name != '/' )
+		long_name = g_strdup_printf( "%s/%s", path, direntry->d_name );
+	      else
+		long_name = direntry->d_name;
+	      plug = custom_plugin_new_with_query( path, direntry->d_name, TRUE );
 	      plug->callbacks = *callbacks;
 #if 0 		
 	      plug->context = 0;
@@ -203,13 +288,21 @@ void plugin_query_all( plugin_callback_struct *callbacks )
 	      plugin_get_all( plug, 1, process_command, NULL ); 
 	      plugin_send_int( plug, 0 );
 #endif
+		if ((pl_list = g_malloc0 (sizeof (plugin_list_data))) == NULL)
+	 	  {
+	    		 g_print ("custom_plugin_query: Memory Allocation Error.\n");
+	   		 return;
+	 	  }
+
+		pl_list->name = g_strdup (direntry->d_name);
+		pl_list->location = g_strdup (path);
+		plugin_list = g_list_append (plugin_list, pl_list);
 	    }
 	}
       closedir( dir );
     }
 }
 
-
 void plugin_finish( plugin *the_plugin )
 {
   close( the_plugin->pipe_to );
@@ -595,16 +688,22 @@ static void plugin_get_more( gpointer data, gint source, GdkInputCondition condi
   if( partly->length - partly->sofar == 0 )
     {
       gdk_input_remove( partly->incall );
-      if( partly->finished )
-	partly->finished( partly->plug, partly->buff, partly->length, partly->data );
-      g_free( partly->buff );
-      g_free( partly );
+      if ( partly->finished )
+        {
+	 partly->finished( partly->plug, partly->buff, partly->length, partly->data );
+/*      if (partly->buff)
+        g_free( partly->buff );
+      if (partly)
+        g_free( partly );*/
+        }
     }
   else if( count == 0 )
     {
       gdk_input_remove( partly->incall );
-      g_free( partly->buff );
-      g_free( partly );
+/*      if (partly->buff)
+        g_free( partly->buff );
+      if (partly)
+        g_free( partly );*/
     }
 }
 
@@ -634,14 +733,18 @@ plugin_send_more( gpointer data, gint source, GdkInputCondition condition )
       gdk_input_remove( partly->incall );
       if( partly->finished )
 	partly->finished( partly->plug, partly->buff, partly->length, partly->data );
-      g_free( partly->buff );
-      g_free( partly );
+/*      if (partly->buff)
+        g_free( partly->buff );
+      if (partly)
+        g_free( partly );*/
     }
   else if( count == -1 )
     {
       gdk_input_remove( partly->incall );
-      g_free( partly->buff );
-      g_free( partly );
+/*      if (partly->buff)
+        g_free( partly->buff );
+      if (partly)
+        g_free( partly );*/
     }
 }
 
@@ -1011,3 +1114,116 @@ process_command( plugin *plug, gchar *buffer, int length, gpointer data )
       plugin_get_all( plug, 1, process_command, NULL );
     }
 }
+
+/* New Gnome Config interface for Plugins.. */
+void plugin_load_list (gchar *app)
+{
+  char *value;
+  char *location;
+  char *c, c2[20];
+  plugin_list_data *pl_list;
+  int i, i2, i3;
+  
+  gnome_config_push_prefix ("/Editor_Plugins/Use/");
+  
+  i3 = gnome_config_get_int (app);
+/*  g_print ("/Editor_Plugins/Use/%s = %d\n", app, i3);*/
+  
+  i = gnome_config_get_int ("Use");
+  if (!i && !i3)
+    {
+      g_print ("plugins_load_list: No Plugins list found..\nUsing defaults..\n");
+      plugin_query_all ( &pl_callbacks);
+      
+      plugin_save_list ();	/* Save the list of plugins now, in case the editor dies
+      					   and the list doesn't get saved... */
+      gnome_config_set_int (app, 1);
+      
+      return;
+    }
+    
+  if (i3 == 1)
+    {
+     for (i2 = 0; i2 < i; i2++)
+        {
+         sprintf (c2, "%d_name", i2);
+         c = g_strdup (c2);
+         value = g_strdup (gnome_config_get_string (c));
+
+         sprintf (c2, "%d_location", i2);
+         c = g_strdup (c2);
+         location = g_strdup (gnome_config_get_string (c));
+       
+         custom_plugin_query (location, value, &pl_callbacks);
+        }
+     }
+     
+   if (i3 == -1)
+     {
+      	/* FIXME: Add plugins to clist.. but dont load them into the menu */
+      for (i2 = 0; i2 < i; i2++)
+         {
+          sprintf (c2, "%d_name", i2);
+          c = g_strdup (c2);
+          value = g_strdup (gnome_config_get_string (c));
+
+          sprintf (c2, "%d_location", i2);
+          c = g_strdup (c2);
+          location = g_strdup (gnome_config_get_string (c));
+       
+		if ((pl_list = g_malloc0 (sizeof (plugin_list_data))) == NULL)
+	  	  {
+	    	   g_print ("custom_plugin_query: Memory Allocation Error.\n");
+	    	   return;
+	  	  }	
+	  
+          	pl_list->name = g_strdup (value);
+		pl_list->location = g_strdup (location);
+		plugin_list = g_list_append (plugin_list, pl_list);
+         }
+     }
+   
+   
+  gnome_config_pop_prefix ();
+  gnome_config_sync ();
+}
+
+void plugin_save_list()
+{
+  char *value;
+  char *location;
+  char *c, c2[20];
+  plugin_list_data *plugins_data;
+  int i, i2;
+  
+  i2 = 0;
+  
+  for (i = 0; i < g_list_length (plugin_list); i++)
+     {
+       i2++;
+     }
+  
+  gnome_config_push_prefix ("/Editor_Plugins/Use/");
+  gnome_config_set_int ("Use", i2);
+  
+  for (i = 0; i < g_list_length (plugin_list); i++)
+     {
+       plugins_data = g_list_nth_data (plugin_list, i);
+       
+
+       value = g_strdup (plugins_data->name);
+       location = g_strdup (plugins_data->location);
+       
+       sprintf (c2, "%i_name", i);
+       c = g_strdup (c2);
+       gnome_config_set_string (c, value);
+       
+       sprintf (c2, "%i_location", i);
+       c = g_strdup (c2);
+       gnome_config_set_string (c, location);
+     }
+  
+  
+  gnome_config_pop_prefix ();
+  gnome_config_sync ();
+}
\ No newline at end of file
